﻿using Hotline.CallCenter.Calls;
using Hotline.Share.Dtos.CallCenter;
using Hotline.Share.Enums;
using Hotline.Share.Requests;
using MapsterMapper;
using Microsoft.AspNetCore.Mvc;

namespace Hotline.Api.Controllers
{
    /// <summary>
    /// 
    /// </summary>
    public class CallController : BaseController
    {
        private readonly ICallDomainService _callDomainService;
        private readonly ICallRepository _callRepository;
        private readonly IMapper _mapper;
        private readonly ICallDetailRepository _callDetailRepository;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="callDomainService"></param>
        /// <param name="callRepository"></param>
        /// <param name="mapper"></param>
        /// <param name="callDetailRepository"></param>
        public CallController(ICallDomainService callDomainService, ICallRepository callRepository, IMapper mapper, ICallDetailRepository callDetailRepository)
        {
            _callDomainService = callDomainService;
            _callRepository = callRepository;
            _mapper = mapper;
            _callDetailRepository = callDetailRepository;
        }


        #region 强拆

        /// <summary>
        /// 强拆分机
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("ClearExt")]
        public async Task ClearExt([FromBody] ClearExtRequest request)
        {
            await _callDomainService.ClearExtAsync(request, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 强拆来电
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("ClearVisitor")]
        public async Task ClearVisitor([FromBody] ClearVisitorRequest request)
        {
            await _callDomainService.ClearVisitorAsync(request, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 强拆去电
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("ClearOuter")]
        public async Task ClearOuter([FromBody] ClearOuterRequest request)
        {
            await _callDomainService.ClearOuterAsync(request, HttpContext.RequestAborted);
        }

        #endregion

        #region 通话记录

        /// <summary>
        /// 分页通话记录列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpGet("paged")]
        public async Task<PagedDto<CallDto>> GetCallList([FromQuery]GetCallListRequest request)
        {
            var (total, items) = await _callRepository.QueryPagedAsync(
                x=> true,
                x => x.OrderByDescending(d => d.CreationTime),
                request.PageIndex,
                request.PageSize,
                (!string.IsNullOrEmpty(request.PhoneNum), d => d.FromNo.Contains(request.PhoneNum!))
            );
            return new PagedDto<CallDto>(total, _mapper.Map<IReadOnlyList<CallDto>>(items));
        }

        #endregion

        #region 对外通话记录

        /// <summary>
        /// 通话记录（外部对接）
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpGet("out-calllist")]
        public async Task<IReadOnlyList<OutCallDto>> GetOutCallList([FromQuery] GetOutCallListRequest request)
        {
            var list = await _callRepository.QueryPartAsync(request.time);
            List<OutCallDto> outCallList = new List<OutCallDto>();
            foreach (var item in list)
            {
                OutCallDto outCallDto = new OutCallDto();
                outCallDto.CallId = item.Id;
                outCallDto.InfoType = Share.Enums.EInfoType.Call;//TODO目前写死（只有电话）
                outCallDto.Direction = item.CallDirection;
                outCallDto.Cpn = item.FromNo??"";
                outCallDto.Cdpn = item.ToNo ?? "";
                outCallDto.Answered = item.CallDetails?.FirstOrDefault(x => x.EventName == "ANSWERED")?.AnswerNo ?? "";
                outCallDto.BeginTime = item.CreationTime;
                outCallDto.OnTime = item.CallDetails?.FirstOrDefault(x => x.EventName == "ANSWERED")?.CreationTime;//TODO 接通时间是否可以为空
                outCallDto.ByeTime = item.CallDetails?.FirstOrDefault(x => x.EventName == "BYE")?.CreationTime;
                outCallDto.TalkTime = item.Duration;
                outCallDto.SoundFileName = item.CallDetails?.FirstOrDefault(x => x.EventName == "BYE")?.Recording ?? "";
                outCallDto.EvaluateResult = "";
                outCallDto.EndBy = item.EndBy;
                outCallDto.OnState = item.CallDetails?.Any(x => x.EventName == "ANSWERED")==true? EOnState.On : EOnState.NoOn;
                outCallList.Add(outCallDto);
            }
            return outCallList;
        }

        #endregion

        #region 监听和强插

        /// <summary>
        /// 监听分机
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("MonitorExt")]
        public async Task MonitorExt([FromBody]MonitorExtRequest request)
        {
            await _callDomainService.MonitorExt(request, HttpContext.RequestAborted);
        }


        /// <summary>
        /// 从监听到插播状态变换
        /// 1. 已知：分机 A 在监听分机 B 与其通话方的通话；
        /// 2. 执行分机 A 的从监听到插播状态变换的 API；
        /// 3. 执行成功时，分机 A 与分机 B 建立通话，分机 B 的原通话方听保持音。
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("MonitorExtToTalk")]
        public async Task MonitorExtToTalk([FromBody]MonitorExtToTalkRequest request)
        {
            await _callDomainService.MonitorExtToTalk(request, HttpContext.RequestAborted);
        }


        /// <summary>
        /// 从插播到监听状态变换
        /// 1. 已知：分机 A 在插播分机 B 的通话；
        /// 2. 执行分机 A 的从插播到监听状态变换的 API；
        /// 3. 执行成功时，分机 A 监听分机 B 及其原通话方的通话。
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("MonitorExtToListen")]
        public async Task MonitorExtToListen([FromBody]MonitorExtToListenRequest request)
        {
            await _callDomainService.MonitorExtToListen(request, HttpContext.RequestAborted);
        }


        #endregion


        #region 强插

        /// <summary>
        /// 强插
        /// 1. 已知：分机 A 当前空闲，分机 B 正在通话中；
        /// 2. 执行分机 A 强插分机 B 的 API；
        /// 3. 执行成功时，分机 A 振铃，摘机后即可形成三方通话。
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("BargeinExt")]
        public async Task BargeinExt([FromBody]BargeinExtRequest request)
        {
            await _callDomainService.BargeinExt(request, HttpContext.RequestAborted);
        }

        #endregion


    }
}
